@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,7 +20,7 @@
 */
 #include "./SDL_internal.h"
 
-#if defined(__WIN32__)
+#if defined(__WIN32__) || defined(__GDK__)
 #include "core/windows/SDL_windows.h"
 #endif
 
@@ -32,7 +32,7 @@
 #include "SDL_assert_c.h"
 #include "video/SDL_sysvideo.h"
 
-#ifdef __WIN32__
+#if defined(__WIN32__) || defined(__GDK__)
 #ifndef WS_OVERLAPPEDWINDOW
 #define WS_OVERLAPPEDWINDOW 0
 #endif
@@ -42,12 +42,21 @@
 #endif
 
 #if defined(__EMSCRIPTEN__)
-#include <emscripten.h>
+    #include <emscripten.h>
+    /* older Emscriptens don't have this, but we need to for wasm64 compatibility. */
+    #ifndef MAIN_THREAD_EM_ASM_PTR
+        #ifdef __wasm64__
+            #error You need to upgrade your Emscripten compiler to support wasm64
+        #else
+            #define MAIN_THREAD_EM_ASM_PTR MAIN_THREAD_EM_ASM_INT
+        #endif
+    #endif
 #endif
 
+/* The size of the stack buffer to use for rendering assert messages. */
+#define SDL_MAX_ASSERT_MESSAGE_STACK 256
 
-static SDL_assert_state SDLCALL
-SDL_PromptAssertion(const SDL_assert_data *data, void *userdata);
+static SDL_assert_state SDLCALL SDL_PromptAssertion(const SDL_assert_data *data, void *userdata);
 
 /*
  * We keep all triggered assertions in a singly-linked list so we can
@@ -63,12 +72,10 @@ static SDL_AssertionHandler assertion_handler = SDL_PromptAssertion;
 static void *assertion_userdata = NULL;
 
 #ifdef __GNUC__
-static void
-debug_print(const char *fmt, ...) __attribute__((format (printf, 1, 2)));
+static void debug_print(const char *fmt, ...) __attribute__((format(printf, 1, 2)));
 #endif
 
-static void
-debug_print(const char *fmt, ...)
+static void debug_print(const char *fmt, ...)
 {
     va_list ap;
     va_start(ap, fmt);
@@ -76,29 +83,42 @@ debug_print(const char *fmt, ...)
     va_end(ap);
 }
 
-
 static void SDL_AddAssertionToReport(SDL_assert_data *data)
 {
     /* (data) is always a static struct defined with the assert macros, so
        we don't have to worry about copying or allocating them. */
     data->trigger_count++;
-    if (data->trigger_count == 1) {  /* not yet added? */
+    if (data->trigger_count == 1) { /* not yet added? */
         data->next = triggered_assertions;
         triggered_assertions = data;
     }
 }
 
+#if defined(__WIN32__) || defined(__GDK__)
+#define ENDLINE "\r\n"
+#else
+#define ENDLINE "\n"
+#endif
+
+static int SDL_RenderAssertMessage(char *buf, size_t buf_len, const SDL_assert_data *data)
+{
+    return SDL_snprintf(buf, buf_len,
+                        "Assertion failure at %s (%s:%d), triggered %u %s:" ENDLINE "  '%s'",
+                        data->function, data->filename, data->linenum,
+                        data->trigger_count, (data->trigger_count == 1) ? "time" : "times",
+                        data->condition);
+}
 
 static void SDL_GenerateAssertionReport(void)
 {
     const SDL_assert_data *item = triggered_assertions;
 
     /* only do this if the app hasn't assigned an assertion handler. */
-    if ((item != NULL) && (assertion_handler != SDL_PromptAssertion)) {
+    if ((item) && (assertion_handler != SDL_PromptAssertion)) {
         debug_print("\n\nSDL assertion report.\n");
         debug_print("All SDL assertions between last init/quit:\n\n");
 
-        while (item != NULL) {
+        while (item) {
             debug_print(
                 "'%s'\n"
                 "    * %s (%s:%d)\n"
@@ -116,7 +136,6 @@ static void SDL_GenerateAssertionReport(void)
     }
 }
 
-
 /* This is not declared in any header, although it is shared between some
     parts of SDL, because we don't want anything calling it without an
     extremely good reason. */
@@ -126,9 +145,8 @@ extern void SDL_ExitProcess(int exitcode);
 #endif
 extern SDL_NORETURN void SDL_ExitProcess(int exitcode);
 
-
 #if defined(__WATCOMC__)
-static void SDL_AbortAssertion (void);
+static void SDL_AbortAssertion(void);
 #pragma aux SDL_AbortAssertion aborts;
 #endif
 static SDL_NORETURN void SDL_AbortAssertion(void)
@@ -137,53 +155,61 @@ static SDL_NORETURN void SDL_AbortAssertion(void)
     SDL_ExitProcess(42);
 }
 
-
-static SDL_assert_state SDLCALL
-SDL_PromptAssertion(const SDL_assert_data *data, void *userdata)
+static SDL_assert_state SDLCALL SDL_PromptAssertion(const SDL_assert_data *data, void *userdata)
 {
-#ifdef __WIN32__
-    #define ENDLINE "\r\n"
-#else
-    #define ENDLINE "\n"
-#endif
-
     const char *envr;
     SDL_assert_state state = SDL_ASSERTION_ABORT;
     SDL_Window *window;
     SDL_MessageBoxData messagebox;
     SDL_MessageBoxButtonData buttons[] = {
-        {   0,  SDL_ASSERTION_RETRY,            "Retry" },
-        {   0,  SDL_ASSERTION_BREAK,            "Break" },
-        {   0,  SDL_ASSERTION_ABORT,            "Abort" },
-        {   SDL_MESSAGEBOX_BUTTON_ESCAPEKEY_DEFAULT,
-                SDL_ASSERTION_IGNORE,           "Ignore" },
-        {   SDL_MESSAGEBOX_BUTTON_RETURNKEY_DEFAULT,
-                SDL_ASSERTION_ALWAYS_IGNORE,    "Always Ignore" }
+        { 0, SDL_ASSERTION_RETRY, "Retry" },
+        { 0, SDL_ASSERTION_BREAK, "Break" },
+        { 0, SDL_ASSERTION_ABORT, "Abort" },
+        { SDL_MESSAGEBOX_BUTTON_ESCAPEKEY_DEFAULT,
+          SDL_ASSERTION_IGNORE, "Ignore" },
+        { SDL_MESSAGEBOX_BUTTON_RETURNKEY_DEFAULT,
+          SDL_ASSERTION_ALWAYS_IGNORE, "Always Ignore" }
     };
-    char *message;
     int selected;
 
-    (void) userdata;  /* unused in default handler. */
+    char stack_buf[SDL_MAX_ASSERT_MESSAGE_STACK];
+    char *message = stack_buf;
+    size_t buf_len = sizeof(stack_buf);
+    int len;
 
-    /* !!! FIXME: why is this using SDL_stack_alloc and not just "char message[SDL_MAX_LOG_MESSAGE];" ? */
-    message = SDL_stack_alloc(char, SDL_MAX_LOG_MESSAGE);
-    if (!message) {
-        /* Uh oh, we're in real trouble now... */
+    (void)userdata; /* unused in default handler. */
+
+    /* Assume the output will fit... */
+    len = SDL_RenderAssertMessage(message, buf_len, data);
+
+    /* .. and if it didn't, try to allocate as much room as we actually need. */
+    if (len >= (int)buf_len) {
+        if (SDL_size_add_overflow(len, 1, &buf_len) == 0) {
+            message = (char *)SDL_malloc(buf_len);
+            if (message) {
+                len = SDL_RenderAssertMessage(message, buf_len, data);
+            } else {
+                message = stack_buf;
+            }
+        }
+    }
+
+    /* Something went very wrong */
+    if (len < 0) {
+        if (message != stack_buf) {
+            SDL_free(message);
+        }
         return SDL_ASSERTION_ABORT;
     }
-    SDL_snprintf(message, SDL_MAX_LOG_MESSAGE,
-                 "Assertion failure at %s (%s:%d), triggered %u %s:" ENDLINE
-                    "  '%s'",
-                 data->function, data->filename, data->linenum,
-                 data->trigger_count, (data->trigger_count == 1) ? "time" : "times",
-                 data->condition);
 
     debug_print("\n\n%s\n\n", message);
 
     /* let env. variable override, so unit tests won't block in a GUI. */
     envr = SDL_getenv("SDL_ASSERT");
-    if (envr != NULL) {
-        SDL_stack_free(message);
+    if (envr) {
+        if (message != stack_buf) {
+            SDL_free(message);
+        }
 
         if (SDL_strcmp(envr, "abort") == 0) {
             return SDL_ASSERTION_ABORT;
@@ -196,7 +222,7 @@ SDL_PromptAssertion(const SDL_assert_data *data, void *userdata)
         } else if (SDL_strcmp(envr, "always_ignore") == 0) {
             return SDL_ASSERTION_ALWAYS_IGNORE;
         } else {
-            return SDL_ASSERTION_ABORT;  /* oh well. */
+            return SDL_ASSERTION_ABORT; /* oh well. */
         }
     }
 
@@ -227,15 +253,13 @@ SDL_PromptAssertion(const SDL_assert_data *data, void *userdata)
         } else {
             state = (SDL_assert_state)selected;
         }
-    }
-
-    else
-    {
+    } else {
 #if defined(__EMSCRIPTEN__)
         /* This is nasty, but we can't block on a custom UI. */
-        for ( ; ; ) {
+        for (;;) {
             SDL_bool okay = SDL_TRUE;
-            char *buf = (char *) EM_ASM_INT({
+            /* *INDENT-OFF* */ /* clang-format off */
+            char *buf = (char *) MAIN_THREAD_EM_ASM_PTR({
                 var str =
                     UTF8ToString($0) + '\n\n' +
                     'Abort/Retry/Ignore/AlwaysIgnore? [ariA] :';
@@ -245,12 +269,14 @@ SDL_PromptAssertion(const SDL_assert_data *data, void *userdata)
                 }
                 return allocate(intArrayFromString(reply), 'i8', ALLOC_NORMAL);
             }, message);
+            /* *INDENT-ON* */ /* clang-format on */
 
             if (SDL_strcmp(buf, "a") == 0) {
                 state = SDL_ASSERTION_ABORT;
-            /* (currently) no break functionality on Emscripten
+#if 0 /* (currently) no break functionality on Emscripten */
             } else if (SDL_strcmp(buf, "b") == 0) {
-                state = SDL_ASSERTION_BREAK; */
+                state = SDL_ASSERTION_BREAK;
+#endif
             } else if (SDL_strcmp(buf, "r") == 0) {
                 state = SDL_ASSERTION_RETRY;
             } else if (SDL_strcmp(buf, "i") == 0) {
@@ -266,13 +292,13 @@ SDL_PromptAssertion(const SDL_assert_data *data, void *userdata)
                 break;
             }
         }
-#elif defined(HAVE_STDIO_H)
+#elif defined(HAVE_STDIO_H) && !defined(__3DS__)
         /* this is a little hacky. */
-        for ( ; ; ) {
+        for (;;) {
             char buf[32];
-            fprintf(stderr, "Abort/Break/Retry/Ignore/AlwaysIgnore? [abriA] : ");
-            fflush(stderr);
-            if (fgets(buf, sizeof (buf), stdin) == NULL) {
+            (void)fprintf(stderr, "Abort/Break/Retry/Ignore/AlwaysIgnore? [abriA] : ");
+            (void)fflush(stderr);
+            if (fgets(buf, sizeof(buf), stdin) == NULL) {
                 break;
             }
 
@@ -293,6 +319,8 @@ SDL_PromptAssertion(const SDL_assert_data *data, void *userdata)
                 break;
             }
         }
+#else
+        SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_WARNING, "Assertion Failed", message, window);
 #endif /* HAVE_STDIO_H */
     }
 
@@ -301,15 +329,14 @@ SDL_PromptAssertion(const SDL_assert_data *data, void *userdata)
         SDL_RestoreWindow(window);
     }
 
-    SDL_stack_free(message);
+    if (message != stack_buf) {
+        SDL_free(message);
+    }
 
     return state;
 }
 
-
-SDL_assert_state
-SDL_ReportAssertion(SDL_assert_data *data, const char *func, const char *file,
-                    int line)
+SDL_assert_state SDL_ReportAssertion(SDL_assert_data *data, const char *func, const char *file, int line)
 {
     SDL_assert_state state = SDL_ASSERTION_IGNORE;
     static int assertion_running = 0;
@@ -317,19 +344,17 @@ SDL_ReportAssertion(SDL_assert_data *data, const char *func, const char *file,
 #ifndef SDL_THREADS_DISABLED
     static SDL_SpinLock spinlock = 0;
     SDL_AtomicLock(&spinlock);
-    if (assertion_mutex == NULL) { /* never called SDL_Init()? */
+    if (!assertion_mutex) { /* never called SDL_Init()? */
         assertion_mutex = SDL_CreateMutex();
-        if (assertion_mutex == NULL) {
+        if (!assertion_mutex) {
             SDL_AtomicUnlock(&spinlock);
-            return SDL_ASSERTION_IGNORE;   /* oh well, I guess. */
+            return SDL_ASSERTION_IGNORE; /* oh well, I guess. */
         }
     }
     SDL_AtomicUnlock(&spinlock);
 
-    if (SDL_LockMutex(assertion_mutex) < 0) {
-        return SDL_ASSERTION_IGNORE;   /* oh well, I guess. */
-    }
-#endif
+    SDL_LockMutex(assertion_mutex);
+#endif /* !SDL_THREADS_DISABLED */
 
     /* doing this because Visual C is upset over assigning in the macro. */
     if (data->trigger_count == 0) {
@@ -341,13 +366,14 @@ SDL_ReportAssertion(SDL_assert_data *data, const char *func, const char *file,
     SDL_AddAssertionToReport(data);
 
     assertion_running++;
-    if (assertion_running > 1) {   /* assert during assert! Abort. */
+    if (assertion_running > 1) { /* assert during assert! Abort. */
         if (assertion_running == 2) {
             SDL_AbortAssertion();
-        } else if (assertion_running == 3) {  /* Abort asserted! */
+        } else if (assertion_running == 3) { /* Abort asserted! */
             SDL_ExitProcess(42);
         } else {
-            while (1) { /* do nothing but spin; what else can you do?! */ }
+            while (1) { /* do nothing but spin; what else can you do?! */
+            }
         }
     }
 
@@ -355,21 +381,20 @@ SDL_ReportAssertion(SDL_assert_data *data, const char *func, const char *file,
         state = assertion_handler(data, assertion_userdata);
     }
 
-    switch (state)
-    {
-        case SDL_ASSERTION_ALWAYS_IGNORE:
-            state = SDL_ASSERTION_IGNORE;
-            data->always_ignore = 1;
-            break;
+    switch (state) {
+    case SDL_ASSERTION_ALWAYS_IGNORE:
+        state = SDL_ASSERTION_IGNORE;
+        data->always_ignore = 1;
+        break;
 
-        case SDL_ASSERTION_IGNORE:
-        case SDL_ASSERTION_RETRY:
-        case SDL_ASSERTION_BREAK:
-            break;  /* macro handles these. */
+    case SDL_ASSERTION_IGNORE:
+    case SDL_ASSERTION_RETRY:
+    case SDL_ASSERTION_BREAK:
+        break; /* macro handles these. */
 
-        case SDL_ASSERTION_ABORT:
-            SDL_AbortAssertion();
-            /*break;  ...shouldn't return, but oh well. */
+    case SDL_ASSERTION_ABORT:
+        SDL_AbortAssertion();
+        /*break;  ...shouldn't return, but oh well. */
     }
 
     assertion_running--;
@@ -381,16 +406,17 @@ SDL_ReportAssertion(SDL_assert_data *data, const char *func, const char *file,
     return state;
 }
 
-
 void SDL_AssertionsQuit(void)
 {
+#if SDL_ASSERT_LEVEL > 0
     SDL_GenerateAssertionReport();
 #ifndef SDL_THREADS_DISABLED
-    if (assertion_mutex != NULL) {
+    if (assertion_mutex) {
         SDL_DestroyMutex(assertion_mutex);
         assertion_mutex = NULL;
     }
 #endif
+#endif /* SDL_ASSERT_LEVEL > 0 */
 }
 
 void SDL_SetAssertionHandler(SDL_AssertionHandler handler, void *userdata)
@@ -413,8 +439,8 @@ void SDL_ResetAssertionReport(void)
 {
     SDL_assert_data *next = NULL;
     SDL_assert_data *item;
-    for (item = triggered_assertions; item != NULL; item = next) {
-        next = (SDL_assert_data *) item->next;
+    for (item = triggered_assertions; item; item = next) {
+        next = (SDL_assert_data *)item->next;
         item->always_ignore = SDL_FALSE;
         item->trigger_count = 0;
         item->next = NULL;
@@ -430,7 +456,7 @@ SDL_AssertionHandler SDL_GetDefaultAssertionHandler(void)
 
 SDL_AssertionHandler SDL_GetAssertionHandler(void **userdata)
 {
-    if (userdata != NULL) {
+    if (userdata) {
         *userdata = assertion_userdata;
     }
     return assertion_handler;
