@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,7 +20,7 @@
 */
 #include "../../SDL_internal.h"
 
-#if SDL_VIDEO_DRIVER_COCOA && SDL_VIDEO_OPENGL_EGL
+#if defined(SDL_VIDEO_DRIVER_COCOA) && defined(SDL_VIDEO_OPENGL_EGL)
 
 #include "SDL_cocoavideo.h"
 #include "SDL_cocoaopengles.h"
@@ -28,12 +28,11 @@
 
 /* EGL implementation of SDL OpenGL support */
 
-int
-Cocoa_GLES_LoadLibrary(_THIS, const char *path)
+int Cocoa_GLES_LoadLibrary(_THIS, const char *path)
 {
     /* If the profile requested is not GL ES, switch over to WIN_GL functions  */
     if (_this->gl_config.profile_mask != SDL_GL_CONTEXT_PROFILE_ES) {
-#if SDL_VIDEO_OPENGL_CGL
+#ifdef SDL_VIDEO_OPENGL_CGL
         Cocoa_GLES_UnloadLibrary(_this);
         _this->GL_LoadLibrary = Cocoa_GL_LoadLibrary;
         _this->GL_GetProcAddress = Cocoa_GL_GetProcAddress;
@@ -49,7 +48,7 @@ Cocoa_GLES_LoadLibrary(_THIS, const char *path)
         return SDL_SetError("SDL not configured with OpenGL/CGL support");
 #endif
     }
-    
+
     if (_this->egl_data == NULL) {
         return SDL_EGL_LoadLibrary(_this, NULL, EGL_DEFAULT_DISPLAY, 0);
     }
@@ -57,14 +56,13 @@ Cocoa_GLES_LoadLibrary(_THIS, const char *path)
     return 0;
 }
 
-SDL_GLContext
-Cocoa_GLES_CreateContext(_THIS, SDL_Window * window)
+SDL_GLContext Cocoa_GLES_CreateContext(_THIS, SDL_Window * window)
 { @autoreleasepool
 {
     SDL_GLContext context;
     SDL_WindowData *data = (__bridge SDL_WindowData *)window->driverdata;
 
-#if SDL_VIDEO_OPENGL_CGL
+#ifdef SDL_VIDEO_OPENGL_CGL
     if (_this->gl_config.profile_mask != SDL_GL_CONTEXT_PROFILE_ES) {
         /* Switch to CGL based functions */
         Cocoa_GLES_UnloadLibrary(_this);
@@ -90,30 +88,25 @@ Cocoa_GLES_CreateContext(_THIS, SDL_Window * window)
     return context;
 }}
 
-void
-Cocoa_GLES_DeleteContext(_THIS, SDL_GLContext context)
+void Cocoa_GLES_DeleteContext(_THIS, SDL_GLContext context)
 { @autoreleasepool
 {
     SDL_EGL_DeleteContext(_this, context);
-    Cocoa_GLES_UnloadLibrary(_this);
 }}
 
-int
-Cocoa_GLES_SwapWindow(_THIS, SDL_Window * window)
+int Cocoa_GLES_SwapWindow(_THIS, SDL_Window * window)
 { @autoreleasepool
 {
     return SDL_EGL_SwapBuffers(_this, ((__bridge SDL_WindowData *) window->driverdata).egl_surface);
 }}
 
-int
-Cocoa_GLES_MakeCurrent(_THIS, SDL_Window * window, SDL_GLContext context)
+int Cocoa_GLES_MakeCurrent(_THIS, SDL_Window * window, SDL_GLContext context)
 { @autoreleasepool
 {
     return SDL_EGL_MakeCurrent(_this, window ? ((__bridge SDL_WindowData *) window->driverdata).egl_surface : EGL_NO_SURFACE, context);
 }}
 
-int
-Cocoa_GLES_SetupWindow(_THIS, SDL_Window * window)
+int Cocoa_GLES_SetupWindow(_THIS, SDL_Window * window)
 {
     NSView* v;
     /* The current context is lost in here; save it and reset it. */
@@ -133,7 +126,7 @@ Cocoa_GLES_SetupWindow(_THIS, SDL_Window * window)
         }
         _this->gl_config.driver_loaded = 1;
     }
-  
+
     /* Create the GLES window surface */
     v = windowdata.nswindow.contentView;
     windowdata.egl_surface = SDL_EGL_CreateSurface(_this, (__bridge NativeWindowType)[v layer]);
