@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997-2013 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,13 +20,12 @@
 #include "SDL.h"
 #include "SDL_revision.h"
 
-int
-main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     SDL_version compiled;
     SDL_version linked;
 
-	/* Enable standard application logging */
+    /* Enable standard application logging */
     SDL_LogSetPriority(SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_PRIORITY_INFO);
 
 #if SDL_VERSION_ATLEAST(2, 0, 0)
@@ -35,13 +34,13 @@ main(int argc, char *argv[])
     SDL_Log("Compiled with SDL older than 2.0\n");
 #endif
     SDL_VERSION(&compiled);
-    SDL_Log("Compiled version: %d.%d.%d.%d (%s)\n",
-           compiled.major, compiled.minor, compiled.patch,
-           SDL_REVISION_NUMBER, SDL_REVISION);
+    SDL_Log("Compiled version: %d.%d.%d (%s)\n",
+            compiled.major, compiled.minor, compiled.patch,
+            SDL_REVISION);
     SDL_GetVersion(&linked);
-    SDL_Log("Linked version: %d.%d.%d.%d (%s)\n",
-           linked.major, linked.minor, linked.patch,
-           SDL_GetRevisionNumber(), SDL_GetRevision());
+    SDL_Log("Linked version: %d.%d.%d (%s)\n",
+            linked.major, linked.minor, linked.patch,
+            SDL_GetRevision());
     SDL_Quit();
-    return (0);
+    return 0;
 }
